//////////////////////////////////////////////////////////////////////
// Stingray Software - Objective Chart(tm) Copyright (c) 1996 Stingray
// Software Inc. All Rights Reserved. This source code is only
// intended as a supplement to the Stingray Objective Chart product.
// See the help files for detailed information regarding the use of these classes
// Written by Bob Powell (www.stingray.com)
/////////////////////////////////////////////////////////////////////

#ifndef __SECCOMDOC__
#define __SECCOMDOC__


#define SEC_NOOP			0x00000000
#define SEC_CLIPBOARD_TEXT	0x00000001
#define SEC_GMEM_INIT		0x00000002
#define SEC_DOC_UPDATE		0x00000003 // broadcast to all owned documents
#define SEC_USER_COMMAND	0x00008000 // ALL commands below 32768 are reserved by possible future SEC++ operations

// SECComDoc is a document class extension with special linking and inter-document communication abilities.
// To use the special functions described here, your document manager must be derived from <c SECDocManager>.<nl><nl>
// Documents based on SECComDoc may own or be parents of other documents. 
// Child documents need not be of the same type as their parents. 
// They may or may not be updated by the parent. 
// All documents, regardless of their child or parent status, will behave like ordinary documents 
// within the application.
// From a developer's point of view, the process of maintaining child lists and notifications is automatic. 
// You simply call the OnNewDocument function with the optional parameters, and child documents 
// derived from SECComDoc will add themselves to the parent's document list for dynamic updating.<nl>
// If a child document is destroyed, it will inform its parent (and all its children) of the fact 
// and its links to the document will be broken automatically.

class SECComDoc : public CDocument
{
// Protected data members
protected:
	/* TRUE while this document is updating the parent document*/
	BOOL m_bUpdatingParent;
	
	/* Position of the current document in the update list*/
	POSITION m_LinkPosition;
	
	/* List of documents that have been opened and may be updated by this document*/
	CPtrList m_DocLinkList;
	
	/* Pointer to the document which opened and updates this document*/
	SECComDoc * m_pParent;

	// protected member functions
protected: // create from serialization only
	/* Constructor*/
	SECComDoc();

	DECLARE_DYNCREATE(SECComDoc)

	//        
	/* Called by the ProcessDocumentCommand routine in response to a SEC_DOC_UPDATE command*/
	virtual BOOL SecDocUpdate(UINT nSubCode,DWORD dwData);
	/* Override this function to process your own commands*/
	virtual BOOL ProcessUserCommand(UINT nOpCode, UINT nSubCode, UINT dwData);
	/* Called in response to a SEC_CLIPBOARD_TEXT command*/
	virtual BOOL ClipboardText(void); // starter function which you must alter to convert between what is on the text clipboard and the internal document structure.
	/* Called in response to a SEG_GMEM_INIT command*/
	virtual BOOL GmemInit(UINT code,HANDLE hMem);


// Attributes
public:
	/* Called to inform the parent document of a change in the child */
	/* the parent may subsequently update its own document and those of its other children as a result. */
	virtual BOOL UpdateParentDocument(UINT nSubCode,DWORD dwData);
	
	/* Retrieves a pointer to the next linked document in the list*/
	virtual SECComDoc * GetNextLink(POSITION& pos);
	
	/* Retrieves a pointer to the previous linked document in the list*/
	virtual SECComDoc * GetPrevLink(POSITION& pos);
	
	/* Retrieves the first document position in the list -- this POSITION may be used to iterate the list.*/
	virtual POSITION GetHeadLinkPosition();
	
	/* Returns the number of documents currently linked to this one*/
	virtual int GetLinkCount();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(SECComDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

/////////////////////////////////////////////////////////////
// SEC++ Special parameter passing system for new documents 
// Overridden OnOpenDocument(...) has expandable features.

// Public member functions
public:
	/* Decodes document commands*/
	virtual BOOL ProcessDocumentCommand(UINT nOpCode, UINT nSubCode, DWORD dwData);
	
	/* Opens a document with OnNewDocument and passes parameters to it*/
	virtual BOOL OnOpenDocument(UINT nOpCode,UINT nSubCode,DWORD dwData);
	
	/* Opens a document with OnNewDocument and passes parameters to it*/
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName); 
	
	/* Sets a document name filtering out document command codes passed as text*/
	virtual void SetPathName(LPCTSTR lpszPathName,BOOL bAddToMRU=TRUE);
	
	/* Called when a child document is deleted*/
	virtual void DestroyingDocument(SECComDoc *doc);
	
	/* Removes the link to a document*/
	virtual void Unlink(SECComDoc *ToUnlink);
	
	/* Notifies a parent document that the child is "alive"*/
	virtual void NotifyLink(SECComDoc *newlink);
	
	/* Initializes a pointer to the parent document*/
	virtual void SetParentPointer(SECComDoc* pParent);
	
	/* Sequentially updates all child documents in the list*/
	virtual BOOL UpdateLinkedDocuments(UINT nSubCode, DWORD dwData=NULL);

// End of SEC++ special functions
//////////////////////////////////////////////////////////////

// Implementation
public:
	/* Destructor*/
	virtual ~SECComDoc();


#ifdef _DEBUG
	/* DEBUG*/
	virtual void AssertValid() const;
	/* DEBUG*/
	virtual void Dump(CDumpContext& dc) const;
#endif

// Generated message map functions
protected:
	//{{AFX_MSG(SECComDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

};


/////////////////////////////////////////////////////////////////////////////

#endif //__SECCOMDOC__

